@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_core.h 22448 2011-05-28 18:49:54Z rouault $
+ * $Id: ogr_core.h 27058 2014-03-19 22:19:21Z kyle $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Define some core portability services for cross-platform OGR code.
@@ -7,6 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 1999, Frank Warmerdam
+ * Copyright (c) 2007-2014, Even Rouault <even dot rouault at mines-paris dot org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -50,12 +51,28 @@ class CPL_DLL OGREnvelope
         OGREnvelope() : MinX(0.0), MaxX(0.0), MinY(0.0), MaxY(0.0)
         {
         }
+
+        OGREnvelope(const OGREnvelope& oOther) :
+            MinX(oOther.MinX),MaxX(oOther.MaxX), MinY(oOther.MinY), MaxY(oOther.MaxY)
+        {
+        }
+
     double      MinX;
     double      MaxX;
     double      MinY;
     double      MaxY;
 
+/* See http://trac.osgeo.org/gdal/ticket/5299 for details on this pragma */
+#if (__GNUC__ == 4 && __GNUC_MINOR__ >= 6 && !defined(_MSC_VER)) 
+#pragma GCC diagnostic ignored "-Wfloat-equal"
+#pragma GCC diagnostic push
+#endif
     int  IsInit() const { return MinX != 0 || MinY != 0 || MaxX != 0 || MaxY != 0; }
+
+#if (__GNUC__ == 4 && __GNUC_MINOR__ >= 6 && !defined(_MSC_VER))
+#pragma GCC diagnostic pop
+#endif
+
     void Merge( OGREnvelope const& sOther ) {
         if( IsInit() )
         {
@@ -149,6 +166,12 @@ class CPL_DLL OGREnvelope3D : public OGREnvelope
         {
         }
 
+        OGREnvelope3D(const OGREnvelope3D& oOther) :
+                            OGREnvelope(oOther),
+                            MinZ(oOther.MinZ), MaxZ(oOther.MaxZ)
+        {
+        }
+
     double      MinZ;
     double      MaxZ;
 
@@ -235,7 +258,7 @@ class CPL_DLL OGREnvelope3D : public OGREnvelope
     {
         return MinX <= other.MinX && MinY <= other.MinY &&
                MaxX >= other.MaxX && MaxY >= other.MaxY &&
-               MaxZ >= other.MaxZ && MaxZ >= other.MaxZ;
+               MinZ <= other.MinZ && MaxZ >= other.MaxZ;
     }
 };
 #else
@@ -276,12 +299,12 @@ typedef int     OGRBoolean;
 /* -------------------------------------------------------------------- */
 /*      ogr_geometry.h related definitions.                             */
 /* -------------------------------------------------------------------- */
+
 /**
  * List of well known binary geometry types.  These are used within the BLOBs
  * but are also returned from OGRGeometry::getGeometryType() to identify the
  * type of a geometry object.
  */
-
 typedef enum 
 {
     wkbUnknown = 0,         /**< unknown type, non-standard */
@@ -307,6 +330,19 @@ typedef enum
     wkbGeometryCollection25D = 0x80000007 /**< 2.5D extension as per 99-402 */
 } OGRwkbGeometryType;
 
+/**
+ * Output variants of WKB we support. 
+ * 99-402 was a short-lived extension to SFSQL 1.1 that used a high-bit flag
+ * to indicate the presence of Z coordiantes in a WKB geometry.
+ * SQL/MM Part 3 and SFSQL 1.2 use offsets of 1000 (Z), 2000 (M) and 3000 (ZM)
+ * to indicate the present of higher dimensional coordinates in a WKB geometry.
+ */
+typedef enum 
+{
+    wkbVariantOgc, /**< Old-style 99-402 extended dimension (Z) WKB types */
+    wkbVariantIso  /**< SFSQL 1.2 and ISO SQL/MM Part 3 extended dimension (Z&M) WKB types */
+} OGRwkbVariant;
+
 #define wkb25DBit 0x80000000
 #define wkbFlatten(x)  ((OGRwkbGeometryType) ((x) & (~wkb25DBit)))
 
@@ -451,9 +487,11 @@ int CPL_DLL OGRParseDate( const char *pszInput, OGRField *psOutput,
 #define OLCFastSetNextByIndex  "FastSetNextByIndex"
 #define OLCStringsAsUTF8       "StringsAsUTF8"
 #define OLCIgnoreFields        "IgnoreFields"
+#define OLCCreateGeomField     "CreateGeomField"
 
 #define ODsCCreateLayer        "CreateLayer"
 #define ODsCDeleteLayer        "DeleteLayer"
+#define ODsCCreateGeomFieldAfterCreateLayer   "CreateGeomFieldAfterCreateLayer"
 
 #define ODrCCreateDataSource   "CreateDataSource"
 #define ODrCDeleteDataSource   "DeleteDataSource"
